"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""


# -*- coding: utf-8 -*-
import citrusleaf as cl
from helpers import convertor
from helpers.caching import CachedMethod
from constants import *
from helpers.info_dict import InfoDict
from datetime import datetime


@CachedMethod()
def get_node_id(ip, port, sock=None):

    return get_node_info(ip, port, sock=sock).get("node", None)


def is_node_available(ip, port, sock=None):

    if get_node_info(ip, port, sock=sock) == {}:
        return False
    return True


def get_node_build(ip, port, sock=None):

    node_build_id = get_node_info(ip, port, sock=sock).get("build", None)
    return node_build_id


@CachedMethod(60)
def get_self_address(ip, port,sock=None):

    node_address = cl.citrusleaf_info(ip, port, "service", sock=sock)
    if node_address == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    return node_address.split(";")[0]  # multiple_interfaces_on_node


def get_cluster_nodes(ip, port, sock=None):

    services = get_node_info(ip, port, sock=sock).get("services", UNABLE_TO_CONNECT)
    service = get_self_address(ip, port, sock=sock)
    node_list = {}
    if services == UNABLE_TO_CONNECT or service == UNABLE_TO_CONNECT:
        return node_list    
    nodes = convertor.text_to_list(services)
    nodes.append(service.split(";")[0]) # multiple_interfaces_on_node    
    for node_address in nodes:
        ip, port = node_address.split(":")
        node_id = get_node_id(ip, port)
        if node_id is not None and node_id not in node_list.keys():
            node_list[node_id] = node_address
    return node_list


def get_node_statistics(ip, port, sock=None):

    statistics = get_node_info(ip, port, clear_cache=True, sock=sock).get("statistics", None)
    if statistics is None:
        return InfoDict(node_status="off")
    statistics = convertor.text_to_list(statistics)
    statistics = convertor.list_to_dict(statistics)
    statistics['timestamp'] = datetime.now().strftime("%Y/%m/%d %H:%M:%S")
    return statistics


@CachedMethod(300)
def get_node_info(ip, port, sock=None):

    info = cl.citrusleaf_info(ip, port, sock=sock)
    if info == UNABLE_TO_CONNECT:
        return {}
    return info


@CachedMethod()
def get_node_configuration(ip, port, sock=None):

    configuration = cl.citrusleaf_info(ip, port, "get-config:", sock=sock)
    if configuration == UNABLE_TO_CONNECT:
        return InfoDict(node_status="off")
    configuration = convertor.text_to_list(configuration)
    return convertor.list_to_dict(configuration)


@CachedMethod()
def get_node_bins_info(ip, port, sock=None):

    bins_info = dict()
    temp = cl.citrusleaf_info(ip, port, "bins", sock=sock)
    if temp == UNABLE_TO_CONNECT:
        return None

    for row in convertor.text_to_list(temp):
        if row:
            namespace, bin_info = convertor.text_to_list(row, ":")
            temp = convertor.text_to_list(bin_info, ",")
            bins_info[namespace] = convertor.list_to_dict(temp[:2])
            bins_info[namespace]["bins"] = temp[2:]

    return bins_info


@CachedMethod()
def get_namespaces(ip, port, sock=None):

    namespaces = cl.citrusleaf_info(ip, port, "namespaces", sock=sock)
    if namespaces == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    return convertor.text_to_set(namespaces)


@CachedMethod()
def get_namespace_statistics(namespace, ip, port, sock=None):

    stats = cl.citrusleaf_info(ip, port, "namespace/%s" % namespace, sock=sock)
    if stats == UNABLE_TO_CONNECT:
        return InfoDict(node_status="off")
    stats = convertor.text_to_list(stats)
    return convertor.list_to_dict(stats)


@CachedMethod()
def get_sets_info(ip, port, sock=None):

    sets_info = dict()
    result = cl.citrusleaf_info(ip, port, "sets", sock=sock)

    if result == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT

    if result:
        for row in convertor.text_to_list(result.strip(), ";"):
            if not row:
                continue

            details = convertor.list_to_dict(
                convertor.text_to_list(row, ":"), "=")
            for key in list(details.keys()):  # For Aerospike 3.0.22+
                if key.startswith('set-'):
                    new_key = key.replace('set-', '')
                    details[new_key] = details.pop(key)

            namespace = sets_info.get(details['ns_name'], [])
            namespace.append(details)
            sets_info[details['ns_name']] = namespace

    return sets_info


@CachedMethod()
def get_dc_statistics(ip, port, dc, sock=None):

    statistics = cl.citrusleaf_info(ip, port, "dc/%s" % dc, sock=sock)
    if statistics == UNABLE_TO_CONNECT:
        return InfoDict()
    statistics = convertor.text_to_list(statistics)
    return convertor.list_to_dict(statistics)


@CachedMethod()
def get_dc_names(ip, port, sock=None):

    names = cl.citrusleaf_info(ip, port, "dcs", sock=sock)
    if names == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT

    return convertor.text_to_list(names)


@CachedMethod()
def get_udf_list(ip, port, sock=None):

    lst = cl.citrusleaf_info(ip, port, "udf-list", sock=sock)
    if lst == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT

    udfs = []
    for record in convertor.text_to_list(lst, ";"):
        if record != '':
            udfs.append(convertor.list_to_dict(
                convertor.text_to_list(record, ",")))

    return udfs


@CachedMethod()
def get_scan_jobs(ip, port, sock=None):

    lst = cl.citrusleaf_info(ip, port, "scan-list", sock=sock)
    if lst == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    elif lst == "No running scans":
        return []

    jobs = []
    for record in convertor.text_to_list(lst):
        jobs.append(convertor.list_to_dict(convertor.text_to_list(record, ":")))

    return jobs


@CachedMethod()
def get_job_details1(ip, port, sock=None):

    lst = cl.citrusleaf_info(ip, port, "mon:cmd=getall-job", sock=sock)
    if lst == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    elif lst == "No running scans" or lst == "" or lst == "error":
        return []

    jobs = {}
    keys = ["trid", "cpu", "mem", "start_time", "run_time", "disk_io_bytes", "net_io_bytes", "priority"]
    for record in lst[1:-1].split("]["):
        details = convertor.text_to_list(record, "|")
        details = dict(zip(keys, details))
        details["address"] = "%s:%s" % (ip, port)
        jobs[details["trid"]] = details

    return jobs


@CachedMethod()
def get_job_details(ip, port, sock=None):

    lst = cl.citrusleaf_info(ip, port, "jobs:", sock=sock)
    if lst == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT
    elif lst == "No running scans" or lst == "" or lst == "error":
        return []

    jobs = {}
    for record in convertor.text_to_list(lst.strip(), ";"):
        details = convertor.list_to_dict(
            convertor.text_to_list(record, ":"), "=")
        details["address"] = "%s:%s" % (ip, port)
        jobs[details["trid"]] = details

    return jobs


@CachedMethod()
def get_index_stats(ip, port, namespace, index, sock=None):

    command = "sindex/%s/%s" % (str(namespace), str(index))
    stats = cl.citrusleaf_info(ip, port, command, sock=sock)
    if stats == UNABLE_TO_CONNECT:
        return InfoDict(node_status="off")
    elif stats == "ns_type=unknown":
        return InfoDict(namespace_found=False)
    elif stats == "Invalid StatsFAIL:201:NO INDEX":
        return InfoDict(index_found=False)
    stats = convertor.text_to_list(stats)
    return convertor.list_to_dict(stats)


@CachedMethod()
def get_indexes(ip, port, sock=None):

    output = cl.citrusleaf_info(ip, port, "sindex", sock=sock)
    if output == UNABLE_TO_CONNECT:
        return UNABLE_TO_CONNECT

    indexes = {}
    output = output.replace("sync_state=:", "sync_state=")
    for record in convertor.text_to_list(output):
        details = convertor.list_to_dict(convertor.text_to_list(record, ":"))
        if "ns" not in details:
            continue
        namespace = indexes.get(details["ns"], [])
        namespace.append(details)
        indexes[details["ns"]] = namespace

    return indexes
